#! /usr/bin/env python

import os
import shutil
import subprocess
import tempfile
import sys
import logging
import argparse

# Configure logging
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S'
)

SPHINX_DIR = os.path.join(os.getcwd(), ".sphinx")

GITHUB_REPO = "canonical/praecepta"
GITHUB_CLONE_URL = f"https://github.com/{GITHUB_REPO}.git"

# Source paths to copy from repo
VALE_FILE_LIST = [
    "styles/Canonical",
    "styles/config/vocabularies/Canonical",
    "styles/config/dictionaries",
    "vale.ini"
]

def clone_repo_and_copy_paths(file_source_dest, overwrite=False):
    """
    Clone the repository to a temporary directory and copy required files
    
    Args:
        file_source_dest: dictionary of file paths to copy from the repository,
            and their destination paths
        overwrite: boolean flag to overwrite existing files in the destination
    
    Returns:
        bool: True if all files were copied successfully, False otherwise
    """

    if not file_source_dest:
        logging.error("No files to copy")
        return False

    # Create temporary directory on disk for cloning
    temp_dir = tempfile.mkdtemp()
    logging.info("Cloning repository <%s> to temporary directory: %s", GITHUB_REPO, temp_dir)
    clone_cmd = ["git", "clone", "--depth", "1", GITHUB_CLONE_URL, temp_dir]

    try:
        result = subprocess.run(
            clone_cmd, 
            capture_output=True, 
            text=True,
            check=True
        )
        logging.debug("Git clone output: %s", result.stdout)
    except subprocess.CalledProcessError as e:
        logging.error("Git clone failed: %s", e.stderr)
        return False

    # Copy files from the cloned repository to the destination paths
    is_copy_success = True
    for source, dest in file_source_dest.items():
        source_path = os.path.join(temp_dir, source)

        if not os.path.exists(source_path):
            is_copy_success = False
            logging.error("Source path not found: %s", source_path)
            continue

        if not copy_files_to_path(source_path, dest, overwrite):
            is_copy_success = False            
            logging.error("Failed to copy %s to %s", source_path, dest)

    # Clean up temporary directory
    logging.info("Cleaning up temporary directory: %s", temp_dir)
    shutil.rmtree(temp_dir)

    return is_copy_success

def copy_files_to_path(source_path, dest_path, overwrite=False):
    """
    Copy a file or directory from source to destination
    
    Args:
        source_path: Path to the source file or directory
        dest_path: Path to the destination
        overwrite: Boolean flag to overwrite existing files in the destination
        
    Returns:
        bool: True if copy was successful, False otherwise
    """
    # Skip if source file doesn't exist
    if not os.path.exists(source_path):
        logging.warning("Source path not found: %s", source_path)
        return False

    logging.info("Copying %s to %s", source_path, dest_path)
    # Handle existing files
    if os.path.exists(dest_path):
        if overwrite:
            logging.info("  Destination exists, overwriting: %s", dest_path)
            if os.path.isdir(dest_path):
                shutil.rmtree(dest_path)
            else:
                os.remove(dest_path)
        else:
            logging.info("  Destination exists, skip copying (use overwrite=True to replace): %s",
                         dest_path)
            return True     # Skip copying

    # Copy the source to destination
    try:
        if os.path.isdir(source_path):
            # entire directory
            shutil.copytree(source_path, dest_path)
        else:
            # individual files
            shutil.copy2(source_path, dest_path)
        return True
    except (shutil.Error, OSError) as e:
        logging.error("Copy failed: %s", e)
        return False

def parse_arguments():
    parser = argparse.ArgumentParser(description="Download Vale configuration files")
    parser.add_argument("--no-overwrite", action="store_true", help="Don't overwrite existing files")
    return parser.parse_args()

def main():
    # Define local directory paths
    vale_files_dict = {file: os.path.join(SPHINX_DIR, file) for file in VALE_FILE_LIST}

    # Parse command line arguments, default to overwrite_enabled = True
    overwrite_enabled = not parse_arguments().no_overwrite

    # Download into /tmp through git clone 
    if not clone_repo_and_copy_paths(vale_files_dict, overwrite=overwrite_enabled):
        logging.error("Failed to download files from repository")
        return 1

    logging.info("Download complete")
    return 0


if __name__ == "__main__":
    sys.exit(main())  # Keep return code



# TiCS: disabled # samba mock

AUTH_SESSION_INFO_DEFAULT_GROUPS = 1 << 0
AUTH_SESSION_INFO_AUTHENTICATED = 1 << 1
AUTH_SESSION_INFO_SIMPLE_PRIVILEGES = 1 << 2

def system_session():
    return

class Session:
    def __init__(self):
        self.security_token = b""

def user_session(samdb, lp_ctx, dn, session_info_flags):
    return Session()



import datetime
import ast
import os
import yaml

# Configuration for the Sphinx documentation builder.
# All configuration specific to your project should be done in this file.
#
# If you're new to Sphinx and don't want any advanced or custom features,
# just go through the items marked 'TODO'.
#
# A complete list of built-in Sphinx configuration values:
# https://www.sphinx-doc.org/en/master/usage/configuration.html
#
# Our starter pack uses the custom Canonical Sphinx extension
# to keep all documentation based on it consistent and on brand:
# https://github.com/canonical/canonical-sphinx


#######################
# Project information #
#######################

# Project name
#
# TODO: Update with the official name of your project or product

project = "ADSys"
author = "Canonical Ltd."


# Sidebar documentation title; best kept reasonably short
#
# TODO: To include a version number, add it here (hardcoded or automated).
#
# TODO: To disable the title, set to an empty string.

html_title = project + " documentation"


# Copyright string; shown at the bottom of the page
#
# Now, the starter pack uses CC-BY-SA as the license
# and the current year as the copyright year.
#
# TODO: If your docs need another license, specify it instead of 'CC-BY-SA'.
#
# TODO: If your documentation is a part of the code repository of your project,
#       it inherits the code license instead; specify it instead of 'CC-BY-SA'.
#
# NOTE: For static works, it is common to provide the first publication year.
#       Another option is to provide both the first year of publication
#       and the current year, especially for docs that frequently change,
#       e.g. 2022–2023 (note the en-dash).
#
#       A way to check a repo's creation date is to get a classic GitHub token
#       with 'repo' permissions; see https://github.com/settings/tokens
#       Next, use 'curl' and 'jq' to extract the date from the API's output:
#
#       curl -H 'Authorization: token <TOKEN>' \
#         -H 'Accept: application/vnd.github.v3.raw' \
#         https://api.github.com/repos/canonical/<REPO> | jq '.created_at'

copyright = "%s CC-BY-SA, %s" % (datetime.date.today().year, author)


# Documentation website URL
#
# TODO: Update with the official URL of your docs or leave empty if unsure.
#
# NOTE: The Open Graph Protocol (OGP) enhances page display in a social graph
#       and is used by social media platforms; see https://ogp.me/

ogp_site_url = "https://documentation.ubuntu.com/adsys/stable/"


# Preview name of the documentation website
#
# TODO: To use a different name for the project in previews, update as needed.

ogp_site_name = project


# Preview image URL
#
# TODO: To customise the preview image, update as needed.

ogp_image = "https://assets.ubuntu.com/v1/253da317-image-document-ubuntudocs.svg"


# Product favicon; shown in bookmarks, browser tabs, etc.

# TODO: To customise the favicon, uncomment and update as needed.

# html_favicon = '.sphinx/_static/favicon.png'


# Dictionary of values to pass into the Sphinx context for all pages:
# https://www.sphinx-doc.org/en/master/usage/configuration.html#confval-html_context

html_context = {
    # Product page URL; can be different from product docs URL
    #
    # TODO: Change to your product website URL,
    #       dropping the 'https://' prefix, e.g. 'ubuntu.com/lxd'.
    #
    # TODO: If there's no such website,
    #       remove the {{ product_page }} link from the page header template
    #       (usually .sphinx/_templates/header.html; also, see README.rst).
    "product_page": "github.com/ubuntu/adsys",
    # Product tag image; the orange part of your logo, shown in the page header
    #
    # TODO: To add a tag image, uncomment and update as needed.
    # 'product_tag': '_static/tag.png',
    # Your Discourse instance URL
    #
    # TODO: Change to your Discourse instance URL or leave empty.
    #
    # NOTE: If set, adding ':discourse: 123' to an .rst file
    #       will add a link to Discourse topic 123 at the bottom of the page.
    "discourse": "",
    # Your Mattermost channel URL
    #
    # TODO: Change to your Mattermost channel URL or leave empty.
    "mattermost": "",
    # Your Matrix channel URL
    #
    # TODO: Change to your Matrix channel URL or leave empty.
    "matrix": "",
    # Your documentation GitHub repository URL
    #
    # TODO: Change to your documentation GitHub repository URL or leave empty.
    #
    # NOTE: If set, links for viewing the documentation source files
    #       and creating GitHub issues are added at the bottom of each page.
    "github_url": "https://github.com/ubuntu/adsys",
    # Docs branch in the repo; used in links for viewing the source files
    #
    # TODO: To customise the branch, uncomment and update as needed.
    "repo_default_branch": "main",
    # Docs location in the repo; used in links for viewing the source files
    #
    # TODO: To customise the directory, uncomment and update as needed.
    "repo_folder": "/docs/",
    # TODO: To enable or disable the Previous / Next buttons at the bottom of pages
    # Valid options: none, prev, next, both
    "sequential_nav": "both",
    # TODO: To enable listing contributors on individual pages, set to True
    "display_contributors": True,
    # Required for feedback button
    "github_issues": "enabled",
}

# TODO: To enable the edit button on pages, uncomment and change the link to a
# public repository on GitHub or Launchpad. Any of the following link domains
# are accepted:
# - https://github.com/example-org/example"
# - https://launchpad.net/example
# - https://git.launchpad.net/example
#
html_theme_options = {
    "source_edit_link": "https://github.com/ubuntu/adsys",
}

# Project slug; see https://meta.discourse.org/t/what-is-category-slug/87897
#
# TODO: If your documentation is hosted on https://docs.ubuntu.com/,
#       uncomment and update as needed.

slug = "adsys"

#######################
# Sitemap configuration: https://sphinx-sitemap.readthedocs.io/
#######################

# Base URL of RTD hosted project

html_baseurl = "https://documentation.ubuntu.com/adsys/"

# URL scheme. Add language and version scheme elements manually e.g. '{0}/{1}/{{link}}'.format(os.environ['READTHEDOCS_LANGUAGE'], os.environ['READTHEDOCS_VERSION'])

# When configured with RTD variables, check for RTD environment so manual runs succeed:

if "READTHEDOCS_VERSION" in os.environ:
    version = os.environ["READTHEDOCS_VERSION"]
    sitemap_url_scheme = "{version}{link}"
else:
    sitemap_url_scheme = "stable/{link}"


# Template and asset locations

html_static_path = [".sphinx/_static"]
templates_path = [".sphinx/_templates"]


#############
# Redirects #
#############

# To set up redirects: https://documatt.gitlab.io/sphinx-reredirects/usage.html
# For example: 'explanation/old-name.html': '../how-to/prettify.html',

# To set up redirects in the Read the Docs project dashboard:
# https://docs.readthedocs.io/en/stable/guides/redirects.html

# NOTE: If undefined, set to None, or empty,
#       the sphinx_reredirects extension will be disabled.

redirects = {
    "tutorial/certificates-auto-enrolment": "../../how-to/certificates-auto-enrollment",
    "tutorial/certificates-auto-enrollment": "../../how-to/certificates-auto-enrollment",
    "tutorials/certificates-auto-enrolment": "../../how-to/certificates-auto-enrollment",
    "tutorials/certificates-auto-enrollment": "../../how-to/certificates-auto-enrollment",
    "how-to/certificates-auto-enrolment": "../../how-to/certificates-auto-enrollment",
}


###########################
# Link checker exceptions #
###########################

# A regex list of URLs that are ignored by 'make linkcheck'
#
# TODO: Remove or adjust the ACME entry after you update the contributing guide

linkcheck_ignore = [
    "http://127.0.0.1:8000",
    "https://leonelson.com/2011/08/15/how-to-increase-your-csr-key-size-on-microsoft-iis-without-removing-the-production-certificate/",
    "https://manpages.ubuntu.com/manpages/man8/*",
    "https://www.samba.org/*",  # giving erroneous link failures as of June 16th 2025
    "https://wiki.samba.org/*",  # giving erroneous link failures as of June 16th 2025
]

# A regex list of URLs where anchors are ignored by 'make linkcheck'

linkcheck_anchors_ignore_for_url = [r"https://github\.com/.*"]

# give linkcheck multiple tries on failure
# linkcheck_timeout = 30
linkcheck_retries = 3

########################
# Configuration extras #
########################

# Custom MyST syntax extensions; see
# https://myst-parser.readthedocs.io/en/latest/syntax/optional.html
#
# NOTE: By default, the following MyST extensions are enabled:
#       substitution, deflist, linkify

# myst_enable_extensions = set()


# Custom Sphinx extensions; see
# https://www.sphinx-doc.org/en/master/usage/extensions/index.html

# NOTE: The canonical_sphinx extension is required for the starter pack.
#       It automatically enables the following extensions:
#       - custom-rst-roles
#       - myst_parser
#       - notfound.extension
#       - related-links
#       - sphinx_copybutton
#       - sphinx_design
#       - sphinx_reredirects
#       - sphinx_tabs.tabs
#       - sphinxcontrib.jquery
#       - sphinxext.opengraph
#       - terminal-output
#       - youtube-links

extensions = [
    "canonical_sphinx",
    "sphinxcontrib.cairosvgconverter",
    "sphinx_last_updated_by_git",
    "sphinx.ext.intersphinx",
    "sphinxcontrib.mermaid",
    "sphinx_sitemap",
]

# Excludes files or directories from processing

exclude_patterns = ["diagrams/readme.md"]

# Adds custom CSS files, located under 'html_static_path'

html_css_files = ["pro_block.css", "cookie-banner.css"]

# Adds custom JavaScript files, located under 'html_static_path'

html_js_files = ["js/bundle.js"]


# Specifies a reST snippet to be appended to each .rst file

rst_epilog = """
.. include:: /reuse/links.txt
.. include:: /reuse/substitutions.txt
"""

# Feedback button at the top; enabled by default
#
# TODO: To disable the button, uncomment this.

# disable_feedback_button = True


# Your manpage URL
#
# TODO: To enable manpage links, uncomment and replace {codename} with required
#       release, preferably an LTS release (e.g. noble). Do *not* substitute
#       {section} or {page}; these will be replaced by sphinx at build time
#
# NOTE: If set, adding ':manpage:' to an .rst file
#       adds a link to the corresponding man section at the bottom of the page.

# manpages_url = 'https://manpages.ubuntu.com/manpages/{codename}/en/' + \
# 'man{section}/{page}.{section}.html'


# Specifies a reST snippet to be prepended to each .rst file
# This defines a :center: role that centers table cell content.
# This defines a :h2: role that styles content for use with PDF generation.

rst_prolog = """
.. role:: center
   :class: align-center
.. role:: h2
    :class: hclass2
.. role:: woke-ignore
    :class: woke-ignore
.. role:: vale-ignore
    :class: vale-ignore
"""

# Workaround for https://github.com/canonical/canonical-sphinx/issues/34

if "discourse_prefix" not in html_context and "discourse" in html_context:
    html_context["discourse_prefix"] = html_context["discourse"] + "/t/"

# Workaround for substitutions.yaml

if os.path.exists("./reuse/substitutions.yaml"):
    with open("./reuse/substitutions.yaml", "r") as fd:
        myst_substitutions = yaml.safe_load(fd.read())

# Add configuration for intersphinx mapping

intersphinx_mapping = {
    "starter-pack": (
        "https://canonical-example-product-documentation.readthedocs-hosted.com/en/latest",
        None,
    )
}

############################################################
### Additional configuration
############################################################


## Add any configuration that is not covered by the common conf.py file.
def run_before_build(app):
    import subprocess

    subprocess.run(
        [
            "./make_toctree.sh",
            "reference/policies/Computer Policies",
            "reference/policies/User Policies",
        ]
    )


def setup(app):
    app.connect("builder-inited", run_before_build)



import argparse
import base64
import logging
import os
import types

import xml.etree.ElementTree as etree

from samba.dcerpc import preg
from samba.dcerpc import misc
from samba.gp_parse.gp_pol import GPPolParser
from pathlib import Path

def main():
    parser = argparse.ArgumentParser(description='Convert XML files to POL using Samba (or the other way around)')
    parser.add_argument('path', type=str,
                        help='Root path to search for XML files (will recurse into subdirectories), or exact path to convert')
    parser.add_argument('-r', '--reverse', action='store_true',
                        help='Search for POL files and convert them to XML')
    parser.add_argument('-v', '--verbose', action='store_true',
                        help='Enable verbose logging')
    args = parser.parse_args()

    if args.verbose:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)

    extension = '.pol.xml' if not args.reverse else '.pol'

    if os.path.isfile(args.path):
        pol_files = [args.path]
        args.reverse = args.path.endswith('.pol')
    else:
        pol_files = file_list(args.path, extension)
        logging.info('Found %d %s files' % (len(pol_files), extension))

    for pol_file in pol_files:
        try:
            if args.reverse:
                logging.info('Converting %s to XML' % pol_file)
                convert_to_xml(pol_file)
            else:
                logging.info('Converting %s to POL' % pol_file)
                convert_to_pol(pol_file)
        except Exception as e:
            logging.error('Failed to convert %s' % pol_file)
            logging.exception(e)
            continue

def file_list(path, extension):
    files = []
    for root, _, filenames in os.walk(path):
        for filename in filenames:
            if filename.endswith(extension):
                files.append(os.path.join(root, filename))
    return files

def convert_to_xml(pol_file):
    parser = GPPolParser()
    with open(pol_file, 'rb') as f:
        pol_data = f.read()
    parser.parse(pol_data)

    # We append .xml when we convert from POL
    xml_file = pol_file + '.xml'
    logging.debug('Writing XML to %s' % xml_file)
    parser.write_xml(xml_file)

def convert_to_pol(xml_file):
    # This is a hack to pick up an unreleased Samba fix for properly parsing
    # empty MULTI_SZ values
    def _load_xml(self, root):
        self.pol_file = preg.file()
        self.pol_file.header.signature = root.attrib['signature']
        self.pol_file.header.version = int(root.attrib['version'])
        self.pol_file.num_entries = int(root.attrib['num_entries'])

        entries = []
        for e in root.findall('Entry'):
            entry = preg.entry()
            entry_type = int(e.attrib['type'])

            entry.type = entry_type

            entry.keyname = e.find('Key').text
            value_name = e.find('ValueName').text
            if value_name is None:
                value_name = ''

            entry.valuename = value_name

            if misc.REG_MULTI_SZ == entry_type:
                values = [x.text for x in e.findall('Value')]
                if values == [None]:
                    data = u'\x00'
                else:
                    data = u'\x00'.join(values) + u'\x00\x00'
                entry.data = data.encode('utf-16le')
            elif (misc.REG_NONE == entry_type):
                pass
            elif (misc.REG_SZ == entry_type or
                  misc.REG_EXPAND_SZ == entry_type):
                string_val = e.find('Value').text
                if string_val is None:
                    string_val = ''
                entry.data = string_val
            elif (misc.REG_DWORD == entry_type or
                  misc.REG_DWORD_BIG_ENDIAN == entry_type or
                  misc.REG_QWORD == entry_type):
                entry.data = int(e.find('Value').text)
            else: # REG UNKNOWN or REG_BINARY
                entry.data = base64.b64decode(e.find('Value').text)

            entries.append(entry)

        self.pol_file.entries = entries

    parser = GPPolParser()

    # Override load_xml method with our custom one
    parser.load_xml = types.MethodType(_load_xml, parser)
    with open(xml_file, 'r') as f:
        xml_data = f.read()
    parser.load_xml(etree.fromstring(xml_data.strip()))

    # We remove .xml when we convert from XML
    xml_path = Path(xml_file)
    pol_file = xml_path.as_posix()[:-len(''.join(xml_path.suffixes))] + '.pol'
    logging.debug('Writing POL to %s' % pol_file)
    parser.write_binary(pol_file)


if __name__ == '__main__':
    main()



# TiCS: disabled # samba mock

dom_sid = ""
descriptor = "SECURITY_DESCRIPTOR"

SECINFO_OWNER = 1<<0
SECINFO_GROUP = 1<<1
SECINFO_DACL = 1<<2

SEC_STD_READ_CONTROL = 1<<0
SEC_ADS_LIST = 1<<1
SEC_ADS_READ_PROP = 1<<2



# TiCS: disabled # system daemons mocks.

import os
import tempfile

import dbus
import dbus.mainloop.glib
import dbusmock
from dbusmock.templates import systemd


DBUS_SYSTEM_SOCKET_PATH = "/dbus/system_bus_socket"
# For testing purpose
# DBUS_SYSTEM_SOCKET_PATH = "/tmp/system_bus_socket"


def start_system_bus(conf_template: str) -> dbus.Bus:
    """Creates and starts a system bus

    Args:
        conf_template (str): Template to be used as the dbus config.

    Returns:
        dbus.Bus: Session bus created accordingly to the config provided.
    """

    conf = tempfile.NamedTemporaryFile(prefix='dbusmock_cfg')
    conf.write(conf_template.format('system', DBUS_SYSTEM_SOCKET_PATH).encode())

    conf.flush()

    (_, addr) = dbusmock.DBusTestCase.start_dbus(conf=conf.name)
    os.environ['DBUS_SYSTEM_BUS_ADDRESS'] = addr
    return dbusmock.DBusTestCase.get_dbus(True)


def run_system_mocks(bus: dbus.Bus, mode: str):
    """Starts the system dbus mocks.

    Args:
        bus (dbus.Bus): Bus in which the mocks will run.
        mode (str): Mode used to configure some of the mocks.
    """
    systemd_on_bus(bus, mode)
    sssd_on_bus(bus)
    ubuntu_advantage_on_bus(bus, mode)

    if not mode == "no_proxy_object":
        ubuntu_proxy_manager_on_bus(bus, mode)


def systemd_on_bus(bus: dbus.Bus, mode: str):
    """ Installs systemd mock on dbus and sets up the adsys scripts and refresh timer services """
    service = dbus.service.BusName(systemd.BUS_NAME,
                                   bus,
                                   allow_replacement=False,
                                   replace_existing=False,
                                   do_not_queue=True)
    main_object = dbusmock.mockobject.DBusMockObject(service, systemd.PATH_PREFIX,
                                                     systemd.MAIN_IFACE, {},
                                                     "/tmp/systemd-mock.log",
                                                     False)
    main_object.AddTemplate("systemd", "")

    # startup time and adsys timer
    startup_time = dbus.UInt64(1621860927000000)
    next_refresh_time = dbus.UInt64(86400000000)
    if mode == "no_startup_time":
        startup_time = ""
    elif mode == "invalid_startup_time":
        startup_time = dbus.String("invalid")
    elif mode == "no_nextrefresh_time":
        next_refresh_time = ""
    elif mode == "invalid_nextrefresh_time":
        next_refresh_time = dbus.String("invalid")

    main_object.AddProperty(
        systemd.MAIN_IFACE, "GeneratorsStartTimestamp", startup_time)

    main_object.AddObject(
        "/org/freedesktop/systemd1/unit/adsys_2dgpo_2drefresh_2etimer",
        "org.freedesktop.systemd1.Timer",
        {
            "NextElapseUSecMonotonic": next_refresh_time,
        },
        [])

    # mock methods not implemented by the dbusmock template
    main_object.AddMethods("", [
        ("EnableUnitFiles", "asbb", "ba(sss)", "ret = [True, [['symlink', '/from/path', '/to/path']]]"),
        ("DisableUnitFiles", "asb", "a(sss)", "ret = [['symlink', '/from/path', '/to/path']]"),
        ("Reload", "", "", "ret = None"),
    ])

    # our script unit
    main_object.AddMockUnit("adsys-machine-scripts.service")


def sssd_on_bus(bus: dbus.Bus):
    """ Installs sssd mock on the bus """
    service = dbus.service.BusName(
        "org.freedesktop.sssd.infopipe",
        bus,
        allow_replacement=True,
        replace_existing=True,
        do_not_queue=True)

    # Create sssd domain, with online and active server status
    main_object = dbusmock.mockobject.DBusMockObject(
        service, "/org/freedesktop/sssd/infopipe/Domains/example_2ecom",
        "org.freedesktop.sssd.infopipe.Domains.Domain", {},
        "/tmp/sssd-mock.log",
        False)
    main_object.AddMethods("", [
        ("IsOnline", "", "b", "ret = True"),
        # In real environments this is the FQDN of a domain controller
        # For testing purposes we need to match the value to the underlying SMB
        # server running on localhost
        ("ActiveServer", "s", "s", 'ret = "localhost:1446"'),
    ])

    main_object.AddObject(
        "/org/freedesktop/sssd/infopipe/Domains/offline",
        "org.freedesktop.sssd.infopipe.Domains.Domain",
        {},
        [
            ("IsOnline", "", "b", "ret = False"),
            ("ActiveServer", "s", "s", 'ret = ""'),
        ])

    main_object.AddObject(
        "/org/freedesktop/sssd/infopipe/Domains/online_no_active_server",
        "org.freedesktop.sssd.infopipe.Domains.Domain",
        {},
        [
            ("IsOnline", "", "b", "ret = True"),
            ("ActiveServer", "s", "s", 'ret = ""'),
        ])


def ubuntu_advantage_on_bus(bus: dbus.bus, mode: str):
    """ Installs ubuntu_advantage mock on the bus """

    # Ubuntu Advantage subscription state
    subscription_state = dbus.Boolean(True)
    if mode == "subscription_disabled":
        subscription_state = dbus.Boolean(False)

    service = dbus.service.BusName(
        "com.canonical.UbuntuAdvantage",
        bus,
        allow_replacement=True,
        replace_existing=True,
        do_not_queue=True)

    dbusmock.mockobject.DBusMockObject(
        service, "/com/canonical/UbuntuAdvantage/Manager",
        "com.canonical.UbuntuAdvantage.Manager",
        {"Attached": subscription_state},
        "/tmp/ubuntu-advantage-mock.log",
        False)

def ubuntu_proxy_manager_on_bus(bus: dbus.bus, mode: str):
    """ Installs ubuntu-proxy-manager mock on the bus """
    service = dbus.service.BusName(
        "com.ubuntu.ProxyManager",
        bus,
        allow_replacement=True,
        replace_existing=True,
        do_not_queue=True)

    call_result = "ret = None"
    if mode == "apply_proxy_fail":
        call_result = """raise dbus.exceptions.DBusException(
            'could not apply proxy settings',
            name='org.freeesktop.DBus.Error.Failed'
        )"""

    # Mock the Apply method
    main_object = dbusmock.mockobject.DBusMockObject(
        service, "/com/ubuntu/ProxyManager",
        "com.ubuntu.ProxyManager", {},
        "/tmp/ubuntu-proxy-manager-mock.log",
        False)
    main_object.AddMethod("", "Apply", "ssssss", "", call_result)
